@@ -1,4 +1,3 @@
-import io
 import argparse
 import math
 import importlib
@@ -9,11 +8,10 @@ import traceback
 import numpy as np
 import torch
 import sympy
-from dataclasses import dataclass, asdict, field, is_dataclass, fields
 import os
-import re
+from dataclasses import dataclass, asdict, field, is_dataclass, fields
 from .execute_util import Rendering, pop_renderings
-from .file_util import ensure_directory_exists, relativize
+from .file_util import relativize
 
 
 @dataclass(frozen=True)
@@ -111,7 +109,7 @@ def parse_directives(line: str) -> list[Directive]:
     return directives
 
 
-def get_inspect_variables(directives: list[Directive]) -> list[str]:
+def get_inspect_expressions(directives: list[Directive]) -> list[str]:
     """
     If code contains "@inspect <variable>" (as a comment), return those variables.
     Example code:
@@ -125,7 +123,7 @@ def get_inspect_variables(directives: list[Directive]) -> list[str]:
     return variables
 
 
-def get_clear_variables(directives: list[Directive]) -> list[str]:
+def get_clear_expressions(directives: list[Directive]) -> list[str]:
     """
     If code contains "@clear <variable>" (as a comment), return the variables to clear.
     Example code:
@@ -176,7 +174,7 @@ def to_serializable_value(value: any) -> Value:
         return Value(type=value_type, contents=str(value))
 
     # Recursive types
-    if isinstance(value, list):
+    if isinstance(value, (tuple, list)):
         return Value(type=value_type, contents=[to_serializable_value(item) for item in value])
     if isinstance(value, dict):
         return Value(type=value_type, contents={to_primitive(k): to_serializable_value(v) for k, v in value.items()})
@@ -186,8 +184,12 @@ def to_serializable_value(value: any) -> Value:
             for field in fields(value)
         })
 
+    # If the class has a designated asdict method, use it
+    if hasattr(value, "asdict"):
+        return Value(type=value_type, contents=to_serializable_value(value.asdict()).contents)
+
     # Force contents to be a string to avoid serialization errors
-    return Value(type=type(value).__name__, contents=str(value))
+    return Value(type=value_type, contents=str(value))
 
 
 def get_type_str(value: any) -> str:
@@ -214,12 +216,10 @@ def execute(module_name: str, inspect_all_variables: bool) -> Trace:
         """Return the last element of `stack`, but skip over items where local_trace_func is active."""
         stack = []
         # stack looks like this:
-        #   <module> execute [good stuff to return] local_trace_func trace_func get_stack
+        #   _run_module_as_main _run_code <module> execute [good stuff to return] local_trace_func trace_func get_stack
         items = traceback.extract_stack()
-        assert items[0].name == "<module>"
-        assert items[1].name == "execute"
-        for item in traceback.extract_stack()[2:]:
-            if item.name in ("trace_func", "local_trace_func", "get_stack"):
+        for item in traceback.extract_stack():
+            if item.name in ("_run_module_as_main", "_run_code", "<module>", "execute", "local_trace_func", "trace_func", "get_stack"):
                 continue
             stack.append(StackElement(
                 path=relativize(item.filename),
@@ -228,7 +228,7 @@ def execute(module_name: str, inspect_all_variables: bool) -> Trace:
                 code=item.line,
             ))
         return stack
-    
+
     def trace_func(frame, event, arg):
         """
         trace_func and local_trace_func are called on various lines of code when executed.
@@ -259,7 +259,7 @@ def execute(module_name: str, inspect_all_variables: bool) -> Trace:
         item = stack[-1]
 
         # Don't step into comprehensions since they're redundant and just stay on the line
-        if item.function_name == "<listcomp>":
+        if item.function_name in ("<listcomp>", "<lambda>"):
             return trace_func
 
         # Handle @stepover (don't recurse)
@@ -305,20 +305,29 @@ def execute(module_name: str, inspect_all_variables: bool) -> Trace:
             # Update the environment with the actual values
             locals = frame.f_locals
             if inspect_all_variables:
-                vars = locals.keys()
+                exprs = locals.keys()
             else:
-                vars = get_inspect_variables(directives)
-            for var in vars:
+                exprs = get_inspect_expressions(directives)
+            for expr in exprs:
+                if "." in expr:  # e.g., node.name
+                    var, attr = expr.split(".", 1)
+                else:
+                    var = expr
+                    attr = None
                 if var in locals:
-                    close_step.env[var] = to_serializable_value(locals[var])
+                    # Follow the attribute chain
+                    value = locals[var]
+                    if attr:
+                        for attr in attr.split("."):
+                            value = getattr(value, attr)
+                    close_step.env[expr] = to_serializable_value(value)
                 else:
                     print(f"WARNING: variable {var} not found in locals")
-                print(f"    env: {var} = {close_step.env.get(var)}")
+                print(f"    env: {expr} = {close_step.env.get(expr)}")
         
-            clear_vars = get_clear_variables(directives)
-            for var in clear_vars:
-                if var in locals:
-                    close_step.env[var] = None
+            clear_exprs = get_clear_expressions(directives)
+            for expr in clear_exprs:
+                close_step.env[expr] = None
 
             # Capture the renderings of the last line
             close_step.renderings = pop_renderings()
@@ -363,7 +372,7 @@ if __name__ == "__main__":
     parser.add_argument("-I", "--inspect-all-variables", help="Inspect all variables (default: only inspect variables mentioned in @inspect comments)", action="store_true")
     args = parser.parse_args()
 
-    ensure_directory_exists(args.output_path)
+    os.makedirs(args.output_path, exist_ok=True)
 
     for module in args.module:
         module = module.replace(".py", "")  # Just in case
