@@ -7,8 +7,8 @@ import os
 import inspect
 import re
 import subprocess
-from file_util import cached, relativize
 from dataclasses import dataclass
+from .file_util import cached, relativize
 from .arxiv_util import is_arxiv_link, arxiv_reference
 from .reference import Reference
 
@@ -26,6 +26,7 @@ class Rendering:
     - text: plain text (verbatim)
     - markdown: to be rendered as markdown
     - image: an image (data = url)
+    - video: a video (data = url)
     - link: an link to internal code or external URL
     """
     type: str
@@ -69,6 +70,22 @@ def image(url: str, style: dict | None = None, width: int | str | None = None):
     _current_renderings.append(Rendering(type="image", data=path, style=style))
 
 
+def video(url: str, style: dict | None = None, width: int | str | None = None):
+    """Show the video at `url`."""
+    style = style or {}
+    if width is not None:
+        style["width"] = width
+
+    if is_url(url):
+        path = cached(url, "video")
+    else:
+        path = url
+        if not os.path.exists(path):
+            raise ValueError(f"Video not found: {path}")
+
+    _current_renderings.append(Rendering(type="video", data=path, style=style))
+
+
 def is_url(url: str) -> bool:
     """Check if `url` looks like a URL."""
     return url.startswith("http")
